clear,clc

% Add folder 'files' to the search path:
addpath('files'); 

% Load the model:
load('model')

% Choose parameter values:
BETA=.96; GAMMA=2; ALPHA=.3; RHO=.8; DELTA=.1; SIGMA=.02;

% Make a vector of parameter values:
params=eval(symparams);

% Make eta
eta=eval(eta);

% Steady state values
kss=((1/BETA-1+DELTA)/ALPHA)^(1/(ALPHA-1));
css=kss^ALPHA-DELTA*kss;
ass=1;

nxss=[log(kss);log(ass)];
nyss=log(css);

% Check that the steady state is correct:

double(subs(f,[yp(:);y(:);xp(:);x(:);symparams(:)],[nyss(:);nyss(:);nxss(:);nxss(:);params(:)]))

% Compute the cross moments:

n_e=size(eta,2); % number of shocks.
M=gaussian_moments(n_e); % if the shocks are independent standard-normal you can use this function to get the cross moments.

% Choose algorithm for the Sylvester equation
% algo='vectorize'; % Simple vectorization.
% algo='dlyap'; % Hessenberg-Schur algorithm.
algo='gensylv'; % Kamenik algorithm

% Compute the perturbation solution:

derivs=solve_dsge(model,params,M,eta,nxss,nyss,approx,algo);

% The structure derivs contains the derivatives of g and h with respect to
% x, where x stacks together the state variables and the perturbation
% parameter. Hence, length(x) equals the numebr of state variables plus 1.
% For more details see Levintal, Oren, "Fifth-Order Perturbation Solution
% to DSGE Models".

% Simulate the model

x0=nxss; % start at the steady state
T=100; % T periods
shocks=randn(n_e,T-1); % shocks from period 2 to T
pruning=0; % no pruning
% pruning=1; % with pruning

[yt,xt]=simul(x0,shocks,nyss,nxss,eta,derivs,approx,pruning,model);