clear,clc

% Add folder 'files' to the search path:
addpath('files'); 

% Load the model:
load('model')

% Choose parameter values:
BETA=.96; GAMMA=2; ALPHA=.3; RHO=.8; DELTA=.1; SIGMA=.02;

% Make a vector of parameter values:
params=eval(symparams);

% Make eta
eta=eval(eta);

% Discretize the shocks.

n_e=size(eta,2); % number of shocks.

[n_nodes,nep,P] = Monomials_2(n_e,eye(n_e)); 
nep=nep'; % transpose to n_e-by-n_nodes

% Steady state values
kss=((1/BETA-1+DELTA)/ALPHA)^(1/(ALPHA-1));
css=kss^ALPHA-DELTA*kss;
ass=1;

nxss=[log(kss);log(ass)];
nyss=log(css);

% build a Smolyak grid

Smol_grid = Smolyak_Grid(model.n_x,order,model.Smolyak_elem_iso); 

Smol_grid=Smol_grid';

grid_points=size(Smol_grid,2);

cheb_c0=nxss; % center of the grid
DELTA=[.2;.2]; % bounds of the grid. 
% The lower and upper boundes of the state variable x(i) are
% cheb_c0(i)-DELTA(i) and cheb_c0(i)+DELTA(i), respectively.

%The unscaled grid:
x_grid=repmat(cheb_c0,1,grid_points)+repmat(DELTA,1,grid_points).*Smol_grid; 

% Take an initial guess for the endogenous variables and transform it into Smolyak polynomials. 
% We assume here a linear function for the initial guess:
% logc=logcss+.9*(logk-logkss)+.9*loga, and logkp=logkss+.9*(logk-logkss)+.9*loga;

% Compute the Smolyak coefficients that interpolate the initial
% guess
gh1=zeros(model.n_y+model.n_x1,grid_points);
for i=1:grid_points
    logkt=x_grid(1,i);
    logat=x_grid(2,i);
    gh1(1,i)=nyss+.9*(logkt-nxss(1))+.9*logat;
    gh1(2,i)=nxss(1)+.9*(logkt-nxss(1))+.9*logat;
end

Tcheb=Tcheb_fun(Smol_grid,[]);

Tcoeffs=Tcheb'\gh1';
Tcoeffs=Tcoeffs';

coeffs=Tcoeffs(:); % this is the initial guess for the Smolyak coefficients

            
% Solve the Smolyak coefficients by the Newton method

% tolerance parameters
tolX=1e-6;
tolF=1e-6;
maxiter=100;

maxload=50; % this parameter controls the size of vectorized variables. Reduce it to reduce memory consumption.

[ ncoeffs,R,J,iter] = smol_newton( coeffs,model,params,x_grid,cheb_c0,DELTA,nep,P,eta,maxload,tolX,tolF,maxiter );

% Evaluate the solution at point x0

x0=nxss+.1;
Tcheb=Tcheb_fun((x0-cheb_c0)./DELTA,[]);
vars=reshape(ncoeffs,model.n_f,[])*Tcheb;

logc=vars(1)
logk=vars(2)
