
clear,clc, close all
load('models') 

% models=models(2); %take a specific model

%%% all solutions
Orders_for_Perturbation=[1,2,3,4,5];
Orders_for_TaylorProjection=[1,2,3];
Orders_for_Smolyak=[1,2,3];

%%% only cheap solutions
if exist('options.mat','file')~=0
    load('options');
else
    do_all_solutions=1;
end
if do_all_solutions==0
    Orders_for_Perturbation=[1,2,3];
    Orders_for_TaylorProjection=[1,2,3];
    Orders_for_Smolyak=[1,2];
end

do_accuracy=1; % compute model residuals over the ergodic set
do_simulation=1; % simulate the model by the different solutions

main_folder=pwd;
addpath(main_folder)

all_versions={'ver1'}; % do disaster parameterizaion

for veri=1:length(all_versions);
    version=all_versions{veri};
    load(version)

    for modeli=1:length(models)
        mkdir(['performance\' version '\' models{modeli}])

        % Take the model equations for accuracy tests from this path:
        addpath([main_folder '\files_for_TaylorProjection\' models{modeli} '\order2\files']);
        load([main_folder '\files_for_TaylorProjection\' models{modeli} '\order2\model'],'model');
        load([main_folder '\all_models\' models{modeli} '\TaylorProjection\choose']);

        n_y=model.n_y;
        n_x=model.n_x;
        n_f=model.n_f;

        % Approximate the ergodic set by tp3
        load([main_folder '\' tp_folder '\' models{modeli} '\order3\solution_tp']);
        rng('default');
        T=10000;
        load([main_folder '\' pert_folder '\' models{modeli} '\order3\perturbation'],'eta_mat','nyss','nxss','params','P1','nep1','n_e','prob_disaster','MUD')

        shocks=[(rand(1,T)<prob_disaster)-MUD;randn(n_e-1,T)];

        N=3; eta=eta_mat; [U3,W3]=create_UW(n_x,3); model.U{3}=U3;model.W{3}=W3;
        [ results,nu ] = fun_simulation( nxss,shocks,coeffs,c0,N,n_y,n_x,n_f,model,params,eta );
        
        % this is the ergodic set
        xt_simul=results(n_y+1:end,:);
        
        % load the basic model specifications
        load([main_folder '\files_for_TaylorProjection\' models{modeli} '\order2\model'],'model','x');
        model0=model;

        n_y=model0.n_y;
        n_x1=model0.n_x1;
        n_x=model0.n_x;
        n_f=model0.n_f;

        % load Taylor Projection
        for order=Orders_for_TaylorProjection
            load([main_folder '\' tp_folder '\' models{modeli} '\order' num2str(order) '\solution_tp_stochastic_ss'],'coeffs','time');
            coeffs=reshape(coeffs,model0.n_f,numel(coeffs)/model0.n_f); coeffs=coeffs(:);
            eval(['tp' num2str(order) '=coeffs;']) 
        end

        T=size(shocks,2);
        nparams=model.n_b;
        tp1_r=zeros(n_f,T);
        tp2_r=zeros(n_f,T);
        tp3_r=zeros(n_f,T);

        et=shocks;
        c0=nxss;
        model=model0;
        
        [~,W2]=create_UW(n_x,2);
        model.W{2}=W2;
        [~,W3]=create_UW(n_x,3);
        model.W{3}=W3;
        
        if do_accuracy==1
            P=P1;
            nep=nep1;
            for t=1:T
                t
                x0=xt_simul(:,t);
                
                % Taylor Projection
                poly.type='power';
                for order=Orders_for_TaylorProjection
                    eval(['coeffs=tp' num2str(order) '(:);'])
                    resids=eval_eqs(coeffs,x0,model,params,eta,c0,nep,P,order,poly);
                    eval(['tp' num2str(order) '_r(:,t)=resids;'])
                end
            end
                save([main_folder '\performance\' version '\' models{modeli} '\accuracy_and_runtime_stochastic_ss'],'tp*','shocks','T')
            else
                load([main_folder '\performance\' version '\' models{modeli} '\accuracy_and_runtime_stochastic_ss'])
        end


        takevars=[1:n_y];
        
        rmpath([main_folder '\files_for_TaylorProjection\' models{modeli} '\order2\files']);

    end
end

