% This file produces impulse response functions of a disaster shock.

clear,clc, close all
load('models') %,models=models(1:2);
main_folder=pwd;
addpath(main_folder)

all_versions={'ver1'}; % choose disaster parameterization
models=models(8); % take model 8

for veri=1:length(all_versions);
    version=all_versions{veri};
    load(version)

    for modeli=1:length(models)
        % Take the model equations for accuracy tests from this path:
        addpath([main_folder '\files_for_TaylorProjection\' models{modeli} '\order2\files']);

        load([main_folder '\all_models\' models{modeli} '\TaylorProjection\choose']);

        % load some parameters
        load([main_folder '\' pert_folder '\' models{modeli} '\order3\perturbation'],'eta_mat','nyss','nxss','params','P1','nep1','n_e','prob_disaster','MUD')
        nep=nep1;P=P1;
        load([main_folder '\files_for_TaylorProjection\' models{modeli} '\order2\model'],'model','y','x','subsvars','symparams');
        model0=model;

        n_y=model0.n_y;
        n_x1=model0.n_x1;
        n_x=model0.n_x;
        n_f=model0.n_f;
        for k=2:5
            [U,W]=create_UW(n_x,k);
            model.U{k}=U;
            model.W{k}=W;
        end

        c0=nxss;
        eta=eta_mat;
        
        % load solutions
        load([main_folder '\performance\' version '\' models{modeli} '\accuracy_and_runtime'],'pert*','tp*','smol*','shocks','T')

        %Impulse response functions
        impulse=1;

        if impulse==1
            % define a disaster shock in period 100 and simulate to period 120
            ir_shocks=[zeros(size(shocks,1),120)];
            ir_shocks(1,:)=-MUD;
            ir_shocks(1,100)=1-MUD;

            % Taylor projection
            poly.type='power';
            
            coeffs=tp1(:); order=1;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,order,n_y,n_x,n_f,model,params,eta );
            ir_tp1=results;
            nu_tp1=nu;
            tp1_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,order,poly);

            coeffs=tp2(:); order=2;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,order,n_y,n_x,n_f,model,params,eta );
            ir_tp2=results;
            nu_tp2=nu;
            tp2_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,order,poly);

            coeffs=tp3(:); order=3;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,order,n_y,n_x,n_f,model,params,eta );
            ir_tp3=results;
            nu_tp3=nu;
            tp3_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,order,poly);


        % perturbation
            coeffs=pert1(:); order=1;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,order,n_y,n_x,n_f,model,params,eta );
            ir_pert1=results;
            nu_pert1=nu;
            pert1_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,order,poly);

            coeffs=pert2(:); order=2;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,order,n_y,n_x,n_f,model,params,eta );
            ir_pert2=results;
            nu_pert2=nu;
            pert2_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,order,poly);

            coeffs=pert3(:); order=3;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,order,n_y,n_x,n_f,model,params,eta );
            ir_pert3=results;
            nu_pert3=nu;
            pert3_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,order,poly);

            coeffs=pert4(:); order=4;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,order,n_y,n_x,n_f,model,params,eta );
            ir_pert4=results;
            nu_pert4=nu;
            pert4_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,order,poly);

            coeffs=pert5(:); order=5;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,order,n_y,n_x,n_f,model,params,eta );
            ir_pert5=results;
            nu_pert5=nu;
            pert5_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,order,poly);

        % Smolyak
            coeffs=smol1(:); order=1; 
            smol_c0=smol1_c0;
            poly.type='smol';
            poly.DELTA=smol1_DELTA;
            poly.Tcheb_fun_folder=[main_folder '\files_for_smolyak\' models{modeli} '\order' num2str(order) '\files'];
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,smol_c0,order,n_y,n_x,n_f,model,params,eta,poly );
            ir_smol1=results;
            nu_smol1=nu;
            smol1_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,smol_c0,nep,P,order,poly);

            coeffs=smol2(:); order=2; 
            smol_c0=smol2_c0;
            poly.type='smol';
            poly.DELTA=smol2_DELTA;
            poly.Tcheb_fun_folder=[main_folder '\files_for_smolyak\' models{modeli} '\order' num2str(order) '\files'];
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,smol_c0,order,n_y,n_x,n_f,model,params,eta,poly );
            ir_smol2=results;
            nu_smol2=nu;
            smol2_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,smol_c0,nep,P,order,poly);

            coeffs=smol3(:); order=3; 
            smol_c0=smol3_c0;
            poly.type='smol';
            poly.DELTA=smol3_DELTA;
            poly.Tcheb_fun_folder=[main_folder '\files_for_smolyak\' models{modeli} '\order' num2str(order) '\files'];
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,smol_c0,order,n_y,n_x,n_f,model,params,eta,poly );
            ir_smol3=results;
            nu_smol3=nu;
            smol3_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,smol_c0,nep,P,order,poly);

            save('ir','ir_*','nu_*')
        else
            load('ir')
        end

        % make the plots
        tvec=95:120; T=length(tvec);
        names={'pert1','pert2','pert3','pert4','pert5','tp1','tp2','tp3','smol1','smol2','smol3'};

        for i=1:length(names)
            disp(names{i})
            eval(['results=ir_' names{i} ';'])
            eval(['nu=nu_' names{i} ';'])
            tildiv=nu(logical(subsvars==sym('tildiv')),:);
            loghatz=nu(logical(subsvars==sym('loghatz')),:);
            logz=cumsum(loghatz);
            logtilqe=results(logical(y==sym('logtilqe')),:);
            logqf=results(logical(y==sym('logqf')),:);
            logtily=nu(logical(subsvars==sym('logtily')),:);
            logtilc=nu(logical(subsvars==sym('logtilc')),:);
            logtilx=results(logical(y==sym('logtilx')),:);
            logy=logtily+logz;
            logx=logtilx+logz;
            logc=logtilc+logz;
            RoE=exp(loghatz(2:end)).*(tildiv(2:end)+exp(logtilqe(2:end))).*exp(-logtilqe(1:end-1));
            RoE=100*(RoE.^4-1); %annualized
            Rf=exp(-logqf);
            Rf=100*(Rf.^4-1);%annualized
            dlogy=logtily(2:end)-logtily(1:end-1)+loghatz(2:end);
            Growth=exp(4*dlogy)-1;

            eval(['RoE_' names{i} '=RoE(tvec(1):end);'])
            eval(['Rf_' names{i} '=Rf(tvec(1):end);'])
            eval(['Growth_' names{i} '=Growth(tvec(1):end);'])
            eval(['logtily_' names{i} '=logtily(tvec(1):end);'])
            eval(['logy_' names{i} '=logy(tvec(1):end);'])
            eval(['logtilx_' names{i} '=logtilx(tvec(1):end);'])
            eval(['logx_' names{i} '=logx(tvec(1):end);'])
            eval(['logtilc_' names{i} '=logtilc(tvec(1):end);'])
            eval(['logc_' names{i} '=logc(tvec(1):end);'])
            eval(['logtilqe_' names{i} '=logtilqe(tvec(1):end);'])
        end
        close all
        tvec2=tvec(1:end-1);
        time=1:length(tvec);
        %Output

        set(0, 'defaultFigurePaperPosition',  [0 0 20 21.5]*20);
        h= figure('color', [1 1 1], 'PaperType', 'A4');

        subplot(3,2,1);
        plot(time,logy_tp3,time,logy_pert1,time,logy_pert3,time,logy_pert5);
        title('log(output)')
        legend('tp3','pert1','pert3','pert5','location','NorthEast')

        subplot(3,2,2)
        plot(time,logy_tp3,time,logy_tp2,time,logy_tp1,time,logy_smol2,time,logy_smol3);
        title('log(output)')
        legend('tp3','tp2','tp1','smol2','smol3','location','NorthEast')

        subplot(3,2,3)
        plot(time,logx_tp3,time,logx_pert1,time,logx_pert3,time,logx_pert5);
        title('log(investment)')
        legend('tp3','pert1','pert3','pert5','location','NorthEast')

        subplot(3,2,4)
        plot(time,logx_tp3,time,logx_tp2,time,logx_tp1,time,logx_smol2,time,logx_smol3);
        title('log(investment)')
        legend('tp3','tp2','tp1','smol2','smol3','location','NorthEast')

        subplot(3,2,5)
        plot(time,logc_tp3,time,logc_pert1,time,logc_pert3,time,logc_pert5);
        title('log(consumption)')
        legend('tp3','pert1','pert3','pert5','location','NorthEast')

        subplot(3,2,6)
        plot(time,logc_tp3,time,logc_tp2,time,logc_tp1,time,logc_smol2,time,logc_smol3);
        title('log(consumption)')
        legend('tp3','tp2','tp1','smol2','smol3','location','NorthEast')

        saveas(h,['newIR_disaster8.eps'],'epsc')

        rmpath([main_folder '\files_for_TaylorProjection\' models{modeli} '\order2\files']);

    end
end

