% This file produces impulse response functions of a disaster risk shock.

clear,clc, close all
load('models') 
main_folder=pwd;
addpath(main_folder)

all_versions={'ver1'}; % choose disaster parameterization
models=models(8); % take model 8

for veri=1:length(all_versions);
    version=all_versions{veri};
    load(version)


    for modeli=1:length(models)
        addpath([main_folder '\files_for_TaylorProjection\' models{modeli} '\order2\files']);

        load([main_folder '\all_models\' models{modeli} '\TaylorProjection\choose']);

        load([main_folder '\' pert_folder '\' models{modeli} '\order3\perturbation'],'eta_mat','nyss','nxss','params','P1','nep1','n_e','prob_disaster','MUD')
        nep=nep1;P=P1;
        load([main_folder '\files_for_TaylorProjection\' models{modeli} '\order2\model'],'model','y','x','subsvars','symparams');
        model0=model;

        n_y=model0.n_y;
        n_x1=model0.n_x1;
        n_x=model0.n_x;
        n_f=model0.n_f;
        for k=2:5
            [U,W]=create_UW(n_x,k);
            model.U{k}=U;
            model.W{k}=W;
        end

        c0=nxss;
        eta=eta_mat;
        load([main_folder '\performance\' version '\' models{modeli} '\accuracy_and_runtime'],'pert*','tp*','smol*','shocks','T')

        %Impulse response
        impulse=1;

        if impulse==1
            ir_shocks=[zeros(size(shocks,1),120)];
            ir_shocks(1,:)=-MUD;
            ir_shocks(2,100)=log(.6/.55)/params(logical(symparams==sym('SDV_THETA'))); %number of sd to disaster risk shock

        % Taylor projection
            poly.type='power';
            
            coeffs=tp1(:); N=1;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,N,n_y,n_x,n_f,model,params,eta );
            ir_tp1=results;
            nu_tp1=nu;
            tp1_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,N,poly);

            coeffs=tp2(:); N=2;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,N,n_y,n_x,n_f,model,params,eta );
            ir_tp2=results;
            nu_tp2=nu;
            tp2_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,N,poly);

            coeffs=tp3(:); N=3;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,N,n_y,n_x,n_f,model,params,eta );
            ir_tp3=results;
            nu_tp3=nu;
            tp3_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,N,poly);


        % perturbation
            coeffs=pert1(:); N=1;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,N,n_y,n_x,n_f,model,params,eta );
            ir_pert1=results;
            nu_pert1=nu;
            pert1_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,N,poly);

            coeffs=pert2(:); N=2;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,N,n_y,n_x,n_f,model,params,eta );
            ir_pert2=results;
            nu_pert2=nu;
            pert2_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,N,poly);

            coeffs=pert3(:); N=3;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,N,n_y,n_x,n_f,model,params,eta );
            ir_pert3=results;
            nu_pert3=nu;
            pert3_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,N,poly);

            coeffs=pert4(:); N=4;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,N,n_y,n_x,n_f,model,params,eta );
            ir_pert4=results;
            nu_pert4=nu;
            pert4_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,N,poly);

            coeffs=pert5(:); N=5;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,N,n_y,n_x,n_f,model,params,eta );
            ir_pert5=results;
            nu_pert5=nu;
            pert5_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,N,poly);

        % Smolyak
            coeffs=smol1(:); N=1; order=N;
            smol_c0=smol1_c0;
            poly.type='smol';
            poly.DELTA=smol1_DELTA;
            poly.Tcheb_fun_folder=[main_folder '\files_for_smolyak\' models{modeli} '\order' num2str(order) '\files'];
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,smol_c0,N,n_y,n_x,n_f,model,params,eta,poly );
            ir_smol1=results;
            nu_smol1=nu;
            smol1_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,smol_c0,nep,P,N,poly);

            coeffs=smol2(:); N=2; order=N;
            smol_c0=smol2_c0;
            poly.type='smol';
            poly.DELTA=smol2_DELTA;
            poly.Tcheb_fun_folder=[main_folder '\files_for_smolyak\' models{modeli} '\order' num2str(order) '\files'];
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,smol_c0,N,n_y,n_x,n_f,model,params,eta,poly );
            ir_smol2=results;
            nu_smol2=nu;
            smol2_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,smol_c0,nep,P,N,poly);

            coeffs=smol3(:); N=3; order=N;
            smol_c0=smol3_c0;
            poly.type='smol';
            poly.DELTA=smol3_DELTA;
            poly.Tcheb_fun_folder=[main_folder '\files_for_smolyak\' models{modeli} '\order' num2str(order) '\files'];
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,smol_c0,N,n_y,n_x,n_f,model,params,eta,poly );
            ir_smol3=results;
            nu_smol3=nu;
            smol3_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,smol_c0,nep,P,N,poly);

            save('ir','ir_*','nu_*')
        else
            load('ir')
        end

        tvec=95:120; T=length(tvec);
        names={'pert1','pert2','pert3','pert4','pert5','tp1','tp2','tp3','smol1','smol2','smol3'};

        for i=1:length(names)
            disp(names{i})
            eval(['results=ir_' names{i} ';'])
            eval(['nu=nu_' names{i} ';'])
            tildiv=nu(logical(subsvars==sym('tildiv')),:);
            loghatz=nu(logical(subsvars==sym('loghatz')),:);
            logz=cumsum(loghatz);
            logtilqe=results(logical(y==sym('logtilqe')),:);
            logqf=results(logical(y==sym('logqf')),:);
            logtily=nu(logical(subsvars==sym('logtily')),:);
            logtilc=nu(logical(subsvars==sym('logtilc')),:);
            logtilx=results(logical(y==sym('logtilx')),:);
            logy=logtily+logz;
            RoE=exp(loghatz(2:end)).*(tildiv(2:end)+exp(logtilqe(2:end))).*exp(-logtilqe(1:end-1));
            RoE=100*(RoE.^4-1); %annualized
            Rf=exp(-logqf);
            Rf=100*(Rf.^4-1);%annualized
            dlogy=logtily(2:end)-logtily(1:end-1)+loghatz(2:end);
            Growth=exp(4*dlogy)-1;

            eval(['RoE_' names{i} '=RoE(tvec(1):end);'])
            eval(['Rf_' names{i} '=Rf(tvec(1):end);'])
            eval(['Growth_' names{i} '=Growth(tvec(1):end);'])
            eval(['logtily_' names{i} '=logtily(tvec(1):end);'])
            eval(['logy_' names{i} '=logy(tvec(1):end);'])
            eval(['logtilx_' names{i} '=logtilx(tvec(1):end);'])
            eval(['logtilc_' names{i} '=logtilc(tvec(1):end);'])
            eval(['logtilqe_' names{i} '=logtilqe(tvec(1):end);'])
        end
        close all
        tvec2=tvec(1:end-1);
        
        time=1:length(tvec);

        set(0, 'defaultFigurePaperPosition',  [0 0 20 21.5]*20);
        h= figure('color', [1 1 1], 'PaperType', 'A4');

        subplot(3,2,1);
        plot(time,logtily_tp3,'b',time,logtily_pert1,time,logtily_pert3,time,logtily_pert5);
        title('log(detrended output)')
        legend('tp3','pert1','pert3','pert5','location','SouthEast')

        subplot(3,2,2)
        plot(time,logtily_tp3,'b',time,logtily_tp2,'r',time,logtily_tp1,time,logtily_smol2,'c',time,logtily_smol3,'m');
        title('log(detrended output)')
        legend('tp3','tp2','tp1','smol2','smol3','location','SouthEast')

        subplot(3,2,3)
        plot(time,logtilx_tp3,'b',time,logtilx_pert1,time,logtilx_pert3,time,logtilx_pert5);
        title('log(detrended investment)')
        legend('tp3','pert1','pert3','pert5','location','SouthEast')

        subplot(3,2,4)
        plot(time,logtilx_tp3,'b',time,logtilx_tp2,'r',time,logtilx_tp1,time,logtilx_smol2,'c',time,logtilx_smol3,'m');
        title('log(detrended investment)')
        legend('tp3','tp2','tp1','smol2','smol3','location','SouthEast')

        subplot(3,2,5)
        plot(time,logtilc_tp3,'b',time,logtilc_pert1,time,logtilc_pert3,time,logtilc_pert5);
        title('log(detrended consumption)')
        legend('tp3','pert1','pert3','pert5','location','SouthEast')

        subplot(3,2,6)
        plot(time,logtilc_tp3,'b',time,logtilc_tp2,'r',time,logtilc_tp1,time,logtilc_smol2,'c',time,logtilc_smol3,'m');
        title('log(detrended consumption)')
        legend('tp3','tp2','tp1','smol2','smol3','location','SouthEast')

        saveas(h,['newIR_disaster_risk8.eps'],'epsc')

        rmpath([main_folder '\files_for_TaylorProjection\' models{modeli} '\order2\files']);

    end
end

