% This file produces impulse response functions of small and big disaster risk shocks.

clear,clc, close all
load('models')
main_folder=pwd;
addpath(main_folder)

all_versions={'ver1'}; % choose disaster parameterization
models=models(8); % take model 8

for veri=1:length(all_versions);
    version=all_versions{veri};
    load(version)

    for modeli=1:length(models)
        addpath([main_folder '\files_for_TaylorProjection\' models{modeli} '\order2\files']);

        load([main_folder '\all_models\' models{modeli} '\TaylorProjection\choose']);

        load([main_folder '\' pert_folder '\' models{modeli} '\order3\perturbation'],'eta_mat','nyss','nxss','params','P1','nep1','n_e','prob_disaster','MUD')
        nep=nep1;P=P1;
        load([main_folder '\files_for_TaylorProjection\' models{modeli} '\order2\model'],'model','y','x','subsvars','symparams');
        model0=model;

        n_y=model0.n_y;
        n_x1=model0.n_x1;
        n_x=model0.n_x;
        n_f=model0.n_f;
        for k=2:5
            [U,W]=create_UW(n_x,k);
            model.U{k}=U;
            model.W{k}=W;
        end

        c0=nxss;
        eta=eta_mat;
        load([main_folder '\performance\' version '\' models{modeli} '\accuracy_and_runtime'],'pert*','tp*','smol*','shocks','T')

        %Impulse response

        impulse=1;

        if impulse==1
            ir_shocks=[zeros(size(shocks,1),120)];
            ir_shocks(1,:)=-MUD;
            ir_shocks(2,100)=log(.6/.55)/params(logical(symparams==sym('SDV_THETA'))); %number of sd to disaster risk shock 1.5

        % Taylor projection
            poly.type='power';

            coeffs=tp2(:); order=2;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,order,n_y,n_x,n_f,model,params,eta );
            ir_tp2=results;
            nu_tp2=nu;
            tp2_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,order,poly);

            coeffs=tp3(:); order=3;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,order,n_y,n_x,n_f,model,params,eta );
            ir_tp3=results;
            nu_tp3=nu;
            tp3_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,order,poly);

        % Smolyak

            coeffs=smol2(:); order=2; 
            smol_c0=smol2_c0;
            poly.type='smol';
            poly.DELTA=smol2_DELTA;
            poly.Tcheb_fun_folder=[main_folder '\files_for_smolyak\' models{modeli} '\order' num2str(order) '\files'];
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,smol_c0,order,n_y,n_x,n_f,model,params,eta,poly );
            ir_smol2=results;
            nu_smol2=nu;
            smol2_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,smol_c0,nep,P,order,poly);

            coeffs=smol3(:); order=3; 
            smol_c0=smol3_c0;
            poly.type='smol';
            poly.DELTA=smol3_DELTA;
            poly.Tcheb_fun_folder=[main_folder '\files_for_smolyak\' models{modeli} '\order' num2str(order) '\files'];
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,smol_c0,order,n_y,n_x,n_f,model,params,eta,poly );
            ir_smol3=results;
            nu_smol3=nu;
            smol3_EE=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,smol_c0,nep,P,order,poly);


            %%% bigger shock
            poly.type='power';

            ir_shocks=[zeros(size(shocks,1),120)];
            ir_shocks(1,:)=-MUD;
            ir_shocks(2,100)=log(.6/.5)/params(logical(symparams==sym('SDV_THETA'))); %number of sd to disaster risk shock 1.5

            coeffs=tp2(:); order=2;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,order,n_y,n_x,n_f,model,params,eta );
            ir_tp2_big=results;
            nu_tp2_big=nu;
            tp2_EE_big=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,order,poly);

            coeffs=tp3(:); order=3;
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,c0,order,n_y,n_x,n_f,model,params,eta );
            ir_tp3_big=results;
            nu_tp3_big=nu;
            tp3_EE_big=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,c0,nep,P,order,poly);


        % Smolyak

            coeffs=smol2(:); order=2; 
            smol_c0=smol2_c0;
            poly.type='smol';
            poly.DELTA=smol2_DELTA;
            poly.Tcheb_fun_folder=[main_folder '\files_for_smolyak\' models{modeli} '\order' num2str(order) '\files'];
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,smol_c0,order,n_y,n_x,n_f,model,params,eta,poly );
            ir_smol2_big=results;
            nu_smol2_big=nu;
            smol2_EE_big=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,smol_c0,nep,P,order,poly);

            coeffs=smol3(:); order=3; 
            smol_c0=smol3_c0;
            poly.type='smol';
            poly.DELTA=smol3_DELTA;
            poly.Tcheb_fun_folder=[main_folder '\files_for_smolyak\' models{modeli} '\order' num2str(order) '\files'];
            [ results,nu ] = fun_simulation( nxss,ir_shocks,coeffs,smol_c0,order,n_y,n_x,n_f,model,params,eta,poly );
            ir_smol3_big=results;
            nu_smol3_big=nu;
            smol3_EE_big=eval_eqs(coeffs,results(n_y+1:end,100),model,params,eta,smol_c0,nep,P,order,poly);

            save('ir','ir_*','nu_*')
        else
            load('ir')
        end

        tvec=95:120; T=length(tvec);
        names={'tp2','tp3','smol2','smol3'};

        for i=1:length(names)
            disp(names{i})
            eval(['results=ir_' names{i} ';'])
            eval(['nu=nu_' names{i} ';'])
            tildiv=nu(logical(subsvars==sym('tildiv')),:);
            loghatz=nu(logical(subsvars==sym('loghatz')),:);
            logz=cumsum(loghatz);
            logtilqe=results(logical(y==sym('logtilqe')),:);
            logqf=results(logical(y==sym('logqf')),:);
            logtily=nu(logical(subsvars==sym('logtily')),:);
            logtilc=nu(logical(subsvars==sym('logtilc')),:);
            logtilx=results(logical(y==sym('logtilx')),:);
            logy=logtily+logz;
            RoE=exp(loghatz(2:end)).*(tildiv(2:end)+exp(logtilqe(2:end))).*exp(-logtilqe(1:end-1));
            RoE=100*(RoE.^4-1); %annualized
            Rf=exp(-logqf);
            Rf=100*(Rf.^4-1);%annualized
            dlogy=logtily(2:end)-logtily(1:end-1)+loghatz(2:end);
            Growth=exp(4*dlogy)-1;

            eval(['RoE_' names{i} '=RoE(tvec(1):end);'])
            eval(['Rf_' names{i} '=Rf(tvec(1):end);'])
            eval(['Growth_' names{i} '=Growth(tvec(1):end);'])
            eval(['logtily_' names{i} '=logtily(tvec(1):end);'])
            eval(['logy_' names{i} '=logy(tvec(1):end);'])
            eval(['logtilx_' names{i} '=logtilx(tvec(1):end);'])
            eval(['logtilc_' names{i} '=logtilc(tvec(1):end);'])
            eval(['logtilqe_' names{i} '=logtilqe(tvec(1):end);'])

            % big shock
            eval(['results=ir_' names{i} '_big;'])
            eval(['nu=nu_' names{i} '_big;'])
            tildiv=nu(logical(subsvars==sym('tildiv')),:);
            loghatz=nu(logical(subsvars==sym('loghatz')),:);
            logz=cumsum(loghatz);
            logtilqe=results(logical(y==sym('logtilqe')),:);
            logqf=results(logical(y==sym('logqf')),:);
            logtily=nu(logical(subsvars==sym('logtily')),:);
            logtilc=nu(logical(subsvars==sym('logtilc')),:);
            logtilx=results(logical(y==sym('logtilx')),:);
            logy=logtily+logz;
            RoE=exp(loghatz(2:end)).*(tildiv(2:end)+exp(logtilqe(2:end))).*exp(-logtilqe(1:end-1));
            RoE=100*(RoE.^4-1); %annualized
            Rf=exp(-logqf);
            Rf=100*(Rf.^4-1);%annualized
            dlogy=logtily(2:end)-logtily(1:end-1)+loghatz(2:end);
            Growth=exp(4*dlogy)-1;


            eval(['RoE_' names{i} '_big=RoE(tvec(1):end);'])
            eval(['Rf_' names{i} '_big=Rf(tvec(1):end);'])
            eval(['Growth_' names{i} '_big=Growth(tvec(1):end);'])
            eval(['logtily_' names{i} '_big=logtily(tvec(1):end);'])
            eval(['logy_' names{i} '_big=logy(tvec(1):end);'])
            eval(['logtilx_' names{i} '_big=logtilx(tvec(1):end);'])
            eval(['logtilc_' names{i} '_big=logtilc(tvec(1):end);'])
            eval(['logtilqe_' names{i} '_big=logtilqe(tvec(1):end);'])        
        end
        close all
        tvec2=tvec(1:end-1);

        
        time=1:length(tvec);

        set(0, 'defaultFigurePaperPosition',  [0 0 20 21.5]*20);
        h= figure('color', [1 1 1], 'PaperType', 'A4');

        subplot(3,2,1)
        plot(time,logtily_tp2,'r',time,logtily_tp3,'b',time,logtily_smol2,'c',time,logtily_smol3,'m');
        title('log(detrended output)')
        legend('tp2','tp3','smol2','smol3','location','SouthEast')

        subplot(3,2,2)
        plot(time,logtily_tp2_big,'r',time,logtily_tp3_big,'b',time,logtily_smol2_big,'c',time,logtily_smol3_big,'m');
        title('log(detrended output)')
        legend('tp2','tp3','smol2','smol3','location','SouthEast')
        
        subplot(3,2,3)
        plot(time,logtilx_tp2,'r',time,logtilx_tp3,'b',time,logtilx_smol2,'c',time,logtilx_smol3,'m');
        title('log(detrended investment)')
        legend('tp2','tp3','smol2','smol3','location','SouthEast')

        subplot(3,2,4)
        plot(time,logtilx_tp2_big,'r',time,logtilx_tp3_big,'b',time,logtilx_smol2_big,'c',time,logtilx_smol3_big,'m');
        title('log(detrended investment)')
        legend('tp2','tp3','smol2','smol3','location','SouthEast')

        subplot(3,2,5)
        plot(time,logtilc_tp2,'r',time,logtilc_tp3,'b',time,logtilc_smol2,'c',time,logtilc_smol3,'m');
        title('log(detrended consumption)')
        legend('tp2','tp3','smol2','smol3','location','SouthEast')

        subplot(3,2,6)
        plot(time,logtilc_tp2_big,'r',time,logtilc_tp3_big,'b',time,logtilc_smol2_big,'c',time,logtilc_smol3_big,'m');
        title('log(detrended consumption)')
        legend('tp2','tp3','smol2','smol3','location','SouthEast')

        saveas(h,['newIR_disaster_risk_big8.eps'],'epsc')

        rmpath([main_folder '\files_for_TaylorProjection\' models{modeli} '\order2\files']);

    end
end

