% Solve by Taylor Projection at the stochastic steady state.

if do_tp==1
    
    mkdir([main_folder '\' tp_folder]);
    tolX=tp_tolX;tolF=tp_tolF;maxiter=tp_maxiter; jacobian_type=tp_jacobian;
    
    for modeli=1:length(models)
%       use 3rd order perturbation for initial guess and parameters
        load([main_folder '\' pert_folder '\' models{modeli} '\order3\perturbation'],'eta_mat','params','P1','nep1','derivs','nyss','nxss','prob_disaster')

        % compute the stochastic steady state by 3rd order perturbation
        pruning=1;
        approx=3;
        shocks=zeros(size(eta_mat,2),100); % zero shocks for 100 periods to get to the stochastic steady state
        shocks(1,:)=-prob_disaster; % disaster shock is zero
        x0=nxss; % start at the nonstochastic steady state
        [yt,xt]=simul(x0,shocks,nyss,nxss,eta_mat,derivs,approx,pruning);
        nxss_stochastic=xt(:,end-1);
        
        cd([main_folder '\all_models\' models{modeli} '\TaylorProjection']);
        do_choose;
        cd(main_folder);
        eta=eta_mat;
        nyss=nyss(choose);
        P=P1;nep=nep1;
        for order=tp_order
            addpath([main_folder '\files_for_TaylorProjection\' models{modeli} '\order' num2str(order) '\files']);
            load([main_folder '\files_for_TaylorProjection\' models{modeli} '\order' num2str(order) '\model'],'model','x')

            n_x=model.n_x;
            n_x1=model.n_x1;
            n_x2=model.n_x2;
            n_y=model.n_y; 
            n_f=model.n_f;

            % Make Initial Guess - third order perturbation solution
            
            [ pert_coeffs ] = GetInitial( order,derivs,choose,nyss,nxss,n_x,n_x1 );


            coeffs=pert_coeffs(:);
            c0=nxss;
            N=order;

            % Run first time to compile files and update model
            model.maxload=tp_maxload;
            model.jacobian=jacobian_type;

            [~,~,model]=tp(coeffs,x0,model,params,eta,c0,nep,P);

            % Solve the model and save
            mkdir([main_folder '\' tp_folder '\' models{modeli} '\order' num2str(order)])
            try
                fprintf(['\n' models{modeli} ': solving Taylor projection, order ' num2str(order) '\n'])
                start=tic;
                [ncoeffs,R,J,iter]=tp_newton(coeffs,nxss_stochastic,model,params,eta,c0,nep,P,order,tolX,tolF,maxiter);
                time=toc(start);
                coeffs=ncoeffs;
            catch % newton failed
                coeffs=NaN*zeros(size(coeffs));
                R=[];
                J=[];
                iter=NaN;
                time=NaN;
            end
            save([main_folder '\' tp_folder '\' models{modeli} '\order' num2str(order)  '\solution_tp_stochastic_ss'],'coeffs','c0','R','J','iter','time','nxss','nxss_stochastic');
           
            rmpath([main_folder '\files_for_TaylorProjection\' models{modeli} '\order' num2str(order) '\files']);
        end
    end
end

cd(main_folder)

